!> \file bmelt-hemin40-regions_mod.f90
!! Module qui calcule la fusion basale (grounded ou ice shelves)
!<

!> \namespace  BMELT_NOR_REGIONS
!! Calcule la fusion basale (grounded ou ice shelves)
!! @note Pour les ice shelves Antarctique, tient compte des diffrentes rgions
!! @note A choisir dans le module_choix
!! \author Christophe
!! \date Octobre 2008
!! @note Used module
!! @note   - use module3D_phy
!<


MODULE  BMELT_NOR_REGIONS

! christophe Octobre 2008
! calcule la fusion basale (grounded ou ice shelves)
! pour les ice shelves du nord, tient compte des diffrentes rgions
! A choisir dans le module_choix

  use module3D_phy
  use printtable

  implicit none


  REAL,dimension(nx,ny) ::  bmgrz       !< fusion basale a la grounding zone
  real,dimension(nx,ny) ::  bmshelf     !< fusion basale sous shelf

!REAL,dimension(nx,ny) :: dist_talu    ! distance du point au talu continental
  INTEGER,dimension(nx,ny) :: typeshelf    !< Type de shelf Ronne->1 Ross ->2 ....
                                      !< utilises pour moduler la fusion sous le shelf
  integer, dimension(10) :: region     !< pour crire dans le fichier param
  character(len=30),dimension(10) :: regname !< nom des rgions 
  real :: bsupshelf

CONTAINS
!-------------------------------------------------------------------------------

!>SUBROUTINE: init_bmelt  
!! Cette routine fait l'initialisation pour la fusion basale.
!<
subroutine init_bmelt

! Cette routine fait l'initialisation pour la fusion basale.
! Elle est appele par inputfile-vec-0.5.f90



! lecture du fichier contenant les distances au talu continental (m)
!      open(88,file=TRIM(DIRNAMEINP)//'distance_talu-40km.xy')
!      do j=1,ny
!         do i=1,nx
!            read(88,'(i3,1x,i3,1x,f10.2)') k,k,dist_talu(i,j)
!         enddo
!      enddo
!      close(88)

! lecture du fichier contenant les types de shelves
!  Pacific ->1,...

!     open(88,file=TRIM(DIRNAMEINP)//'region_ocean-hn40.dat')
      open(88,file=TRIM(DIRNAMEINP)//'region_ocean-2006-hn40.dat')
      typeshelf(:,:)=100
      do k=1,nx*ny
        read(88,*,end=36) i,j,typeshelf(i,j)
      end do
36      close(88)
  
!        print*,'APRES',typeshelf(:,:)
        
 region(:)=0
 regname(1)='Pacific'
 regname(2)='deep Arctic'
 regname(3)='deep Atlantic'
 regname(4)='Udson'
 regname(5)='Shallow Arctic'
 regname(6)='Barents sea'
 regname(7)='Shallow Atlantic'
!regname(8)='Continent' !! YA PAS DE 8 !!
 regname(9)='Continent'
 
 bms_init:    do j=1,ny
    do i=1,nx

!            if ((sealevel-bsoc(i,j)).lt.600) then
!                    bmshelf(i,j)=0.2 ! 0.45!   0.65  ;  bmgrz(i,j)=0.2 ! 0.45 !  0.65
!            else    
!                    
!                    bmshelf(i,j)= 2.45!   0.65  ;   bmgrz(i,j)= 2.45 !  0.65
!            endif  
       if (typeshelf(i,j).eq.1) then ! Pacific
          bmshelf(i,j)=5.45!   0.65  
          bmgrz(i,j)=5.45 !  0.65
          if (region(1).eq.0) then
             region(1)=1
             write(42,80)regname(1),bmshelf(i,j),bmgrz(i,j)
             write(6,80)regname(1),bmshelf(i,j),bmgrz(i,j)
          endif


       else  if (typeshelf(i,j).eq.2) then ! Deep Arctic
          bmshelf(i,j)= 0.25!1.5    
          bmgrz(i,j)=0.25   !1.5     
          if (region(2).eq.0) then
             region(2)=1
             write(42,80)regname(2),bmshelf(i,j),bmgrz(i,j)
             write(6,80)regname(2),bmshelf(i,j),bmgrz(i,j)
          endif
       else  if (typeshelf(i,j).eq.3) then ! Deep Atlantic
          bmshelf(i,j)=5.45  
          bmgrz(i,j)=5.45
          if (region(3).eq.0) then
             region(3)=1
             write(42,80)regname(3),bmshelf(i,j),bmgrz(i,j)
             write(6,80)regname(3),bmshelf(i,j),bmgrz(i,j)
          endif


       else  if (typeshelf(i,j).eq.4) then ! Udson
          bmshelf(i,j)=.3  
          bmgrz(i,j)=.3
          if (region(4).eq.0) then
             region(4)=1
             write(42,80)regname(4),bmshelf(i,j),bmgrz(i,j)
          endif
       else  if (typeshelf(i,j).eq.5) then ! 'Shallow Arctic'
          bmshelf(i,j)=.15  
          bmgrz(i,j)=0.15
          if (region(5).eq.0) then
             region(5)=1
             write(42,80)regname(5),bmshelf(i,j),bmgrz(i,j)
          endif
       else  if (typeshelf(i,j).eq.6) then !  'Barents sea'
          bmshelf(i,j)=0.21  
          bmgrz(i,j)=0.21
          if (region(6).eq.0) then  !'Barents sea'
             region(6)=1
             write(42,80)regname(6),bmshelf(i,j),bmgrz(i,j)
          endif
!               if (region(6).eq.0) then
!                  region(6)=1
!                  write(42,80)regname(6),bmshelf(i,j),bmgrz(i,j)
!               endif

       else  if (typeshelf(i,j).eq.7) then !'Shallow Atlantic'
          bmshelf(i,j)=0.55
          bmgrz(i,j)=0.55  
          if (region(7).eq.0) then  !
             region(7)=1  
             write(42,80)regname(7),bmshelf(i,j),bmgrz(i,j)     
          endif
!           else if (region(7).eq.0) then !'Shallow Atlantic'
!                    bmshelf(i,j)=0.55  
!                    bmgrz(i,j)=0.55
!               if (region(7).eq.0) then
!                  region(7)=1
!                  write(42,80)regname(7),bmshelf(i,j),bmgrz(i,j)
               
!       else if (region(9).eq.0) then !'Continent =eau douce'
       else  if (typeshelf(i,j).eq.9) then !'Shallow Atlantic'
          bmshelf(i,j)=0.1 
          bmgrz(i,j)=0.1
          if (region(9).eq.0) then
             region(9)=1
             write(42,80)regname(9),bmshelf(i,j),bmgrz(i,j)
          endif

       endif
    enddo
 enddo bms_init
80 format(a32,' bmshelf=',f8.4,'  bmgrz=',f8.4)

 nom_table='bmshe'
!cdc call printtable_r(bmshelf,nom_table)
      
!stop
 bsupshelf=.2
! write(32,*)
! write(6,*) "fin subroutine init_bmelt"

 return
end subroutine init_bmelt

!________________________________________________________________________________

!> SUBROUTINE: bmeltshelf
!! Cette routine calcule la fusion basale proprement dite 
!<

subroutine bmeltshelf


! cette routine calcule la fusion basale proprement dite

  integer :: ngr           ! nombre de voisins flottants

  
  do j=2,ny-1
     do i=2,nx-1
 

        shelf: if (flot(i,j)) then    ! partie flottante

!           if ((sealevel-bsoc(i,j)).lt.600) then
!                   if ((j.lt.75).and.(i.lt.145) ) then  !Atlantique 
!                   bmelt(i,j)=(.6+.4*coefbmshelf)*bmshelf(i,j)
!                   else
           bmelt(i,j)=coefbmshelf*bmshelf(i,j)
!                   endif
!           else
!           bmelt(i,j)=bmshelf(i,j)
!           endif

           if (fbm(i,j))  bmelt(i,j)=coefbmshelf*bmgrz(i,j)


!                  if (time.gt.5000.) then
!                     bmelt(i,j)=bmelt(i,j)+bsupshelf
!                  endif    

! ATTENTION LE BLOC SUIVANT SERT A FAIRE DES ICE SHELVES STATIONNAIRES
! igrdline est dfini dans itnitial1

           if (igrdline.eq.1) then
              corrbmelt(i,j)=corrbmelt(i,j)+hdot(i,j)*0.8
              bmelt(i,j)=bmelt(i,j)+corrbmelt(i,j)
           endif


        else                   ! point pos, on compte le nombre de voisins flottants
           ngr=0
           if (flot(i+1,j)) ngr=ngr+1
           if (flot(i-1,j)) ngr=ngr+1
           if (flot(i,j+1)) ngr=ngr+1
           if (flot(i,j-1)) ngr=ngr+1

!   la fusion des points limites est une combinaison entre valeur pose et valeur flottante
!   en fonction du nombre de points flottants

           bmelt(i,j)= ngr/4.*bmgrz(i,j)*coefbmshelf+(1.-ngr/4.)*bmelt(i,j)

           

        endif shelf

     end do
  end do


  return
end subroutine  bmeltshelf



END MODULE  BMELT_NOR_REGIONS
