      module netcdf_mod

      use module3D_phy

      implicit none

#if defined key_anteis
      integer :: nx_min = 0   ! 0  ! 95-1 ! 101-1
      integer :: nx_max = nx  ! nx ! 115  ! 129
      integer :: ny_min = 0   ! 0  ! 39-1 ! 73-1
      integer :: ny_max = ny  ! ny ! 59   ! 100
#endif
#if defined key_heino
      integer :: nx_min = 0    ! 0  ! 40-1
      integer :: nx_max = nx   ! nx ! 61
      integer :: ny_min = 0    ! 0  ! 25-1
      integer :: ny_max = ny   ! ny ! 40
#endif
      end module netcdf_mod


#if defined key_netcdf

      subroutine write_flowlaw(filin1,sa1,s2a1,btt1)

        USE netcdf
        USE io_netcdf
        use netcdf_mod

        USE module3D_phy
        USE module_choix

        implicit none

        real,dimension(NX,NY,NZ,n1poly:n2poly) :: sa1
        real,dimension(NX,NY,NZ,n1poly:n2poly) :: S2A1
        real,dimension(NX,NY,NZ,n1poly:n2poly) :: BTT1

        real ghf1(nx,ny)
        character(len=80) :: filin1  
        INTEGER :: status
        INTEGER :: ncid
        character(len=1000) :: filout1
        REAL*8, DIMENSION(:,:,:), POINTER :: tab1
        CHARACTER(len=20),DIMENSION(3) :: dimnames1


        filout1 ='Netcdf/'//trim(filin1)//'.nc'
        status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
        status  = nf90_close(ncid)

        CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
        CALL Write_Ncdf_dim('y',TRIM(filout1),ny)
        CALL Write_Ncdf_dim('z',TRIM(filout1),nz)

        dimnames1(1)='x'
        dimnames1(2)='y'
        dimnames1(2)='z'

        ALLOCATE(tab1(nx,ny,nz))
        tab1(:,:,:) = sa1(:,:,:,n1poly)

        CALL Write_Ncdf_var('SA',dimnames1,TRIM(filout1),tab1,'double')
        DEALLOCATE(tab1)

        ALLOCATE(tab1(nx,ny,nz))
        tab1(:,:,:) = S2A1(:,:,:,n1poly)

        CALL Write_Ncdf_var('S2A',dimnames1,TRIM(filout1),tab1,'double')
        DEALLOCATE(tab1)
           ALLOCATE(tab1(nx,ny,nz))
           tab1(:,:,:) = BTT1(:,:,:,n1poly)

        CALL Write_Ncdf_var('BTT',dimnames1,TRIM(filout1),tab1,'double')
        DEALLOCATE(tab1)

        return
      end subroutine write_flowlaw
!
      subroutine write_flot(filin1,flotmx1,flotmy1,flot1)

        USE netcdf
        USE io_netcdf
        use netcdf_mod

        USE module3D_phy

        implicit none

        logical,dimension(NX,NY) :: FLOT1
        logical,dimension(NX,NY) :: FLOTMX1
        logical,dimension(NX,NY) :: FLOTMY1
        character(len=80) :: filin1
        real,dimension(NX,NY) :: iFLOT
        real,dimension(NX,NY) :: iFLOTMX
        real,dimension(NX,NY) :: iFLOTMY
        INTEGER :: status
        INTEGER :: ncid
        character(len=1000) :: filout1
        REAL*8, DIMENSION(:,:), POINTER :: tab
        CHARACTER(len=20),DIMENSION(2) :: dimnames


        iflot   = 0.
        iflotmx = 0.
        iflotmy = 0.
        do j = 1 , ny
           do i = 1 , nx
              if ( flot1(i,j) .EQ. .TRUE. ) then
                 iflot(i,j) = 1.
              endif
              if ( flotmx1(i,j) .EQ. .TRUE. ) then
                 iflotmx(i,j) = 1.
              endif
              if ( flotmy1(i,j) .EQ. .TRUE. ) then
                 iflotmy(i,j) = 1.
              endif
           enddo
        enddo

        filout1 ='Netcdf/'//trim(filin1)//'.nc'
        status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
        status  = nf90_close(ncid)

        CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
        CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

        dimnames(1)='x'
        dimnames(2)='y'
        ALLOCATE(tab(nx,ny))
        tab(:,:) = iflot(:,:)
        CALL Write_Ncdf_var('flot',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)
        ALLOCATE(tab(nx,ny))
        tab(:,:) = iflotmx(:,:)
        CALL Write_Ncdf_var('flotmx',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)
        ALLOCATE(tab(nx,ny))
        tab(:,:) = iflotmy(:,:)
        CALL Write_Ncdf_var('flotmy',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)

        return
      end subroutine write_flot


      subroutine write_heino(filin1,ghf1)

        USE netcdf
        USE io_netcdf
        use netcdf_mod

        USE module3D_phy

        implicit none

        real ghf1(nx,ny)
        character(len=80) :: filin1
        INTEGER :: status
        INTEGER :: ncid
        character(len=1000) :: filout1
        REAL*8, DIMENSION(:,:), POINTER :: tab
        CHARACTER(len=20),DIMENSION(2) :: dimnames

        filout1 ='Netcdf/'//trim(filin1)//'.nc'
        status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
        status  = nf90_close(ncid)

        CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
        CALL Write_Ncdf_dim('y',TRIM(filout1),ny)
        dimnames(1)='x'
        dimnames(2)='y'
           ALLOCATE(tab(nx,ny))
           tab(:,:) = ghf1(:,:)
        CALL Write_Ncdf_var('var',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)

        return
      end subroutine write_heino



      subroutine write_grzone(filin1,S_sealev1,H_sealev1,B_sealev1,M_sealev1)

        USE netcdf
        USE io_netcdf
        use netcdf_mod

        USE module3D_phy

        implicit none

        real S_sealev1(nx,ny)
        real H_sealev1(nx,ny)
        real B_sealev1(nx,ny)
        real M_sealev1(nx,ny)
        character(len=80) :: filin1
        INTEGER :: status
        INTEGER :: ncid
        character(len=1000) :: filout1
        REAL*8, DIMENSION(:,:), POINTER :: tab
        CHARACTER(len=20),DIMENSION(2) :: dimnames

        filout1 ='Netcdf/'//trim(filin1)//'.nc'
        status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
        status  = nf90_close(ncid)

        CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
        CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

        dimnames(1)='x'
        dimnames(2)='y'
        ALLOCATE(tab(nx,ny))
        tab(:,:) = S_sealev1(:,:)
        CALL Write_Ncdf_var('S_sealev',dimnames,TRIM(filout1),tab,'double')
        ALLOCATE(tab(nx,ny))
        tab(:,:) = H_sealev1(:,:)
        CALL Write_Ncdf_var('H_sealev',dimnames,TRIM(filout1),tab,'double')
        ALLOCATE(tab(nx,ny))
        tab(:,:) = B_sealev1(:,:)
        CALL Write_Ncdf_var('B_sealev',dimnames,TRIM(filout1),tab,'double')
        ALLOCATE(tab(nx,ny))
        tab(:,:) = M_sealev1(:,:)
        CALL Write_Ncdf_var('M_sealev',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)

        return
      end subroutine write_grzone



      subroutine write_coord(filin1,xcc1,YCC1,xlong1,ylat1)

        USE netcdf
        USE io_netcdf
        use netcdf_mod

        USE module3D_phy

        implicit none

        real xcc1(nx,ny)
        real YCC1(nx,ny)
        real xlong1(nx,ny)
        real ylat1(nx,ny)
        character(len=80) :: filin1
        INTEGER :: status
        INTEGER :: ncid
        character(len=1000) :: filout1
        REAL*8, DIMENSION(:,:), POINTER :: tab
        CHARACTER(len=20),DIMENSION(2) :: dimnames
        filout1 ='Netcdf/'//trim(filin1)//'.nc'
        status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
        CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

        dimnames(1)='x'
        dimnames(2)='y'
        ALLOCATE(tab(nx,ny))
        tab(:,:) = xcc1(:,:)
        CALL Write_Ncdf_var('xcc',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)

        ALLOCATE(tab(nx,ny))
        tab(:,:) = YCC1(:,:)

        CALL Write_Ncdf_var('ycc',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)

        ALLOCATE(tab(nx,ny))
        tab(:,:) = xlong1(:,:)
        CALL Write_Ncdf_var('xlong',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)
        ALLOCATE(tab(nx,ny))
        tab(:,:) = ylat1(:,:)
        CALL Write_Ncdf_var('ylat1',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)

        return
      end subroutine write_coord

      subroutine write_ijphi(filin1,ghf1)

        USE netcdf
        USE io_netcdf
        use netcdf_mod

        USE module3D_phy

        implicit none

        real ghf1(nx,ny)
        character(len=80) :: filin1
        INTEGER :: status
        INTEGER :: ncid
        character(len=1000) :: filout1
        REAL*8, DIMENSION(:,:), POINTER :: tab
        CHARACTER(len=20),DIMENSION(2) :: dimnames

        filout1 ='Netcdf/'//trim(filin1)//'.nc'
        status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
        status  = nf90_close(ncid)


        CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
        CALL Write_Ncdf_dim('y',TRIM(filout1),ny)


        dimnames(1)='x'
        dimnames(2)='y'
        ALLOCATE(tab(nx,ny))
        tab(:,:) = ghf1(:,:)

        CALL Write_Ncdf_var('flux_geothermique_Shapiro',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)

        return
      end subroutine write_ijphi

      subroutine write_coupe(filin1,distcent1)

        USE netcdf
        USE io_netcdf
        use netcdf_mod

        USE module3D_phy

        implicit none

        real distcent1(nx,ny)
        character(len=80) :: filin1
        INTEGER :: status
        INTEGER :: ncid
        character(len=1000) :: filout1
        REAL*8, DIMENSION(:,:), POINTER :: tab
        CHARACTER(len=20),DIMENSION(2) :: dimnames
        filout1 ='Netcdf/'//trim(filin1)//'.nc'
        status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
        status  = nf90_close(ncid)

        CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
        CALL Write_Ncdf_dim('y',TRIM(filout1),ny)


        dimnames(1)='x'
        dimnames(2)='y'
        ALLOCATE(tab(nx,ny))
        tab(:,:) = distcent1(:,:)

     CALL Write_Ncdf_var('distcent',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     return
   end subroutine write_coupe

   subroutine write_lect_ieis(filin1,data)

     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     integer data(nx,ny)
     character(len=80) :: filin1
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     CHARACTER(len=20),DIMENSION(2) :: dimnames

     filout1 ='Netcdf/'//trim(filin1)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     dimnames(1)='x'
     dimnames(2)='y'
     ALLOCATE(tab(nx,ny))
     tab(:,:) = data(:,:)

     CALL Write_Ncdf_var('var',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     return
   end subroutine write_lect_ieis

   subroutine write_lect_eis(filin1,data)

     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     real data(nx,ny)
     character(len=80) :: filin1
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     CHARACTER(len=20),DIMENSION(2) :: dimnames

     filout1 ='Netcdf/'//trim(filin1)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     dimnames(1)='x'
     dimnames(2)='y'

     ALLOCATE(tab(nx,ny))
     tab(:,:) = data(:,:)

     CALL Write_Ncdf_var('var',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     return
   end subroutine write_lect_eis

   subroutine write_lect_heino(filin1,mk01)
     !
     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     integer,dimension(NX,NY) :: mk01
     character(len=80) :: filin1
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     CHARACTER(len=20),DIMENSION(2) :: dimnames

     filout1 ='Netcdf/'//trim(filin1)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     dimnames(1)='x'
     dimnames(2)='y'

     ALLOCATE(tab(nx,ny))
     tab(:,:) = mk0(:,:)
     CALL Write_Ncdf_var('mk0',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     return
   end subroutine write_lect_heino

   subroutine write_linear(filin1)

     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     character(len=80) :: filin1
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:,:), POINTER :: tab1
     CHARACTER(len=20),DIMENSION(3) :: dimnames1

     filout1 ='Netcdf/'//trim(filin1)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     CALL Write_Ncdf_dim('z',TRIM(filout1),nz+nzm)

     dimnames1(1)='x'
     dimnames1(2)='y'
     dimnames1(3)='z'

     ALLOCATE(tab1(nx,ny,nz+nzm))
     tab1(:,:,:) = T(:,:,:)
     CALL Write_Ncdf_var('T',dimnames1,TRIM(filout1),tab1,'double')
     DEALLOCATE(tab1)

     return
   end subroutine write_linear


   subroutine write_hz_output(filout)

     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     character(len=80) :: filout
     CHARACTER(len=10) :: tmp_name
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     CHARACTER(len=10) :: namevar
     CHARACTER(len=20),DIMENSION(2) :: dimnames
     integer :: loc_tab

     filout1 ='Netcdf/'//trim(filout)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     dimnames(1)='x'
     dimnames(2)='y'
     do k = 1 , npos

        ALLOCATE(tab(nx,ny))
        tab(:,:) = xxx(:,:,k)
        namevar = TRIM(ADJUSTL(colname(k)))
        loc_tab = INDEX(namevar,'        ')
        if ( loc_tab == 0 ) loc_tab = LEN_TRIM(namevar)+1
        tmp_name = TRIM(ADJUSTL(namevar(0:loc_tab-1)))
        CALL Write_Ncdf_var(TRIM(tmp_name), &
             dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)
     enddo


     return
   end subroutine write_hz_output


   subroutine write_printtable_i(table_i1,filedebug1)

     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     integer,dimension(nx,ny)  :: table_i1
     CHARACTER (len=15) :: filedebug1
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     CHARACTER(len=20),DIMENSION(2) :: dimnames

     filout1 ='Netcdf/'//trim(filedebug1)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     dimnames(1)='x'
     dimnames(2)='y'
     ALLOCATE(tab(nx,ny))
     tab(:,:) = table_i1(:,:)
     CALL Write_Ncdf_var('table_i',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)


     return
   end subroutine write_printtable_i

      subroutine write_printtable_l(table_l1,filedebug1)

      USE netcdf
      USE io_netcdf
      use netcdf_mod

      USE module3D_phy

      implicit none

      CHARACTER (len=15) :: filedebug1
      integer,dimension(nx,ny)  :: table_l1
      INTEGER :: status
      INTEGER :: ncid
      character(len=1000) :: filout1
      REAL*8, DIMENSION(:,:), POINTER :: tab
      CHARACTER(len=20),DIMENSION(2) :: dimnames
      filout1 ='Netcdf/'//trim(filedebug1)//'.nc'
      status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
      status  = nf90_close(ncid)

         CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
         CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

      dimnames(1)='x'
      dimnames(2)='y'

      ALLOCATE(tab(nx,ny))
       do j=1,ny
          do i=1,nx
             if (table_l1(i,j)) then
                tab(i,j) =   1
             else
                tab(i,j) =   0
             endif
          enddo
        enddo
      CALL Write_Ncdf_var('table_l',dimnames,TRIM(filout1),tab,'double')
      DEALLOCATE(tab)

      return
      end subroutine write_printtable_l

      subroutine write_printtable_r(table_r1,filedebug1)

      USE netcdf
      USE io_netcdf
      use netcdf_mod

      USE module3D_phy

      implicit none

      CHARACTER (len=15) :: filedebug1
      integer,dimension(nx,ny)  :: table_r1
      INTEGER :: status
      INTEGER :: ncid
      character(len=1000) :: filout1
      REAL*8, DIMENSION(:,:), POINTER :: tab
      CHARACTER(len=20),DIMENSION(2) :: dimnames

      filout1 ='Netcdf/'//trim(filedebug1)//'.nc'
      status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
      status  = nf90_close(ncid)


         CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
         CALL Write_Ncdf_dim('y',TRIM(filout1),ny)


      dimnames(1)='x'
      dimnames(2)='y'

         ALLOCATE(tab(nx,ny))
         tab(:,:) = table_r1(:,:)
      CALL Write_Ncdf_var('table_r',dimnames,TRIM(filout1),tab,'double')
      DEALLOCATE(tab)

      return
      end subroutine write_printtable_r

      subroutine write_dragging2(fil_vitbil1)

      USE netcdf
      USE io_netcdf
      use netcdf_mod

      USE module3D_phy

      implicit none

      character(len=80) :: fil_vitbil1

      INTEGER :: status
      INTEGER :: ncid
      character(len=1000) :: filout1
      REAL*8, DIMENSION(:,:), POINTER :: tab
      CHARACTER(len=20),DIMENSION(2) :: dimnames

      filout1 ='Netcdf/'//trim(fil_vitbil1)//'.nc'
      status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
      status  = nf90_close(ncid)


         CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
         CALL Write_Ncdf_dim('y',TRIM(filout1),ny)


      dimnames(1)='x'
      dimnames(2)='y'

      ALLOCATE(tab(nx,ny))
      tab(:,:) = mstream_mx(:,:)

      CALL Write_Ncdf_var('mstream_mx',dimnames,TRIM(filout1),tab,'double')
      DEALLOCATE(tab)

      ALLOCATE(tab(nx,ny))
      tab(:,:) = mstream_my(:,:)

      CALL Write_Ncdf_var('mstream_my',dimnames,TRIM(filout1),tab,'double')
      DEALLOCATE(tab)

      return
      end subroutine write_dragging2

      subroutine write_dragging(fil_vitbil1,uxbil1,uybil1)

        USE netcdf
        USE io_netcdf
        use netcdf_mod

        USE module3D_phy

        implicit none

        integer,dimension(nx,ny) :: uxbil1
        integer,dimension(nx,ny) :: uybil1
        character(len=80) :: fil_vitbil1
        INTEGER :: status
        INTEGER :: ncid
        character(len=1000) :: filout1
        REAL*8, DIMENSION(:,:), POINTER :: tab
        CHARACTER(len=20),DIMENSION(2) :: dimnames


        filout1 ='Netcdf/'//trim(fil_vitbil1)//'.nc'
        status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
        CALL Write_Ncdf_dim('y',TRIM(filout1),ny)


        dimnames(1)='x'
        dimnames(2)='y'

        ALLOCATE(tab(nx,ny))
        tab(:,:) = uxbil1(:,:)
        CALL Write_Ncdf_var('uxbil',dimnames,TRIM(filout1),tab,'double')
        DEALLOCATE(tab)

        ALLOCATE(tab(nx,ny))
        tab(:,:) = uybil1(:,:)
     CALL Write_Ncdf_var('uybil',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     return
   end subroutine write_dragging

   subroutine write_init_bmelt2(filin1)

#if defined key_anteis
     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     character(len=80) :: filin1
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     CHARACTER(len=20),DIMENSION(2) :: dimnames

     filout1 ='Netcdf/'//trim(filin1)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     dimnames(1)='x'
     dimnames(2)='y'

     ALLOCATE(tab(nx,ny))
     tab(:,:) = typeshelf(:,:)
     CALL Write_Ncdf_var('numer_ice_shelves',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

#endif

     return
   end subroutine write_init_bmelt2


   subroutine write_init_bmelt(filin1)

#if defined key_anteis
     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     character(len=80) :: filin1
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     CHARACTER(len=20),DIMENSION(2) :: dimnames

     filout1 ='Netcdf/'//trim(filin1)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)
     dimnames(1)='x'
     dimnames(2)='y'

     ALLOCATE(tab(nx,ny))
     tab(:,:) = dist_talu(:,:)

     CALL Write_Ncdf_var('distance_talu',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

#endif

     return
   end subroutine write_init_bmelt

   subroutine write_sortiecptr(filin1)

     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     character(len=80) :: filin1
     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     REAL*8, DIMENSION(:,:,:), POINTER :: tab1
     CHARACTER(len=20),DIMENSION(2) :: dimnames
     CHARACTER(len=20),DIMENSION(3) :: dimnames1

     filout1 ='Netcdf/'//trim(filin1)//'.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     CALL Write_Ncdf_dim('z',TRIM(filout1),nz)
     CALL Write_Ncdf_dim('z1',TRIM(filout1),nz+nzm)
     ! tab 2D
     dimnames(1)='x'
     dimnames(2)='y'

     ALLOCATE(tab(nx,ny))
     tab(:,:) = S(:,:)

     CALL Write_Ncdf_var('S',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     ALLOCATE(tab(nx,ny))
     tab(:,:) = H(:,:)

     CALL Write_Ncdf_var('H',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = B(:,:)
     CALL Write_Ncdf_var('B',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = Bsoc(:,:)
     CALL Write_Ncdf_var('Bsoc',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = IBase(:,:)
     CALL Write_Ncdf_var('IBase',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = Bmelt(:,:)
     CALL Write_Ncdf_var('Bmelt',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = Hwater(:,:)
     CALL Write_Ncdf_var('Hwater',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     ALLOCATE(tab(nx,ny))
     tab(:,:) = UXBAR(:,:)

     CALL Write_Ncdf_var('UXBAR',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     ALLOCATE(tab(nx,ny))
     tab(:,:) = UYBAR(:,:)

     CALL Write_Ncdf_var('UYBAR',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ! tab 3D
     dimnames1(1)='x'
     dimnames1(2)='y'
     dimnames1(3)='z1'
     ALLOCATE(tab1(nx,ny,nz+nzm))
     tab1(:,:,:) = T(:,:,:)
     CALL Write_Ncdf_var('T',dimnames1,TRIM(filout1),tab1,'double')
     DEALLOCATE(tab1)
     dimnames1(1)='x'
     dimnames1(2)='y'
     dimnames1(3)='z'
     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = UX(:,:,:)
     CALL Write_Ncdf_var('UX',dimnames1,TRIM(filout1),tab1,'double')
     DEALLOCATE(tab1)

     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = UY(:,:,:)

     CALL Write_Ncdf_var('UY',dimnames1,TRIM(filout1),tab1,'double')
     DEALLOCATE(tab1)

     return
   end subroutine write_sortiecptr


   subroutine write_cptrtemp()

     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy

     implicit none

     INTEGER :: status
     INTEGER :: ncid
     character(len=1000) :: filout1
     REAL*8, DIMENSION(:,:), POINTER :: tab
     REAL*8, DIMENSION(:,:,:), POINTER :: tab1
     CHARACTER(len=20),DIMENSION(2) :: dimnames
     CHARACTER(len=20),DIMENSION(3) :: dimnames1


     filout1 ='Netcdf/Restart.nc'
     status  = nf90_create(TRIM(filout1),NF90_WRITE,ncid)
     status  = nf90_close(ncid)

     CALL Write_Ncdf_dim('x',TRIM(filout1),nx)
     CALL Write_Ncdf_dim('y',TRIM(filout1),ny)

     CALL Write_Ncdf_dim('z',TRIM(filout1),nz)
     CALL Write_Ncdf_dim('z1',TRIM(filout1),nz+nzm)
     ! tab 2D
     dimnames(1)='x'
     dimnames(2)='y'

     ALLOCATE(tab(nx,ny))
     tab(:,:) = S(:,:)
     CALL Write_Ncdf_var('S',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)

     ALLOCATE(tab(nx,ny))
     tab(:,:) = H(:,:)
     CALL Write_Ncdf_var('H',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = B(:,:)
     CALL Write_Ncdf_var('B',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = Bsoc(:,:)
     CALL Write_Ncdf_var('Bsoc',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = IBase(:,:)
     CALL Write_Ncdf_var('IBase',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = Bmelt(:,:)
     CALL Write_Ncdf_var('Bmelt',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = Hwater(:,:)
     CALL Write_Ncdf_var('Hwater',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = UXBAR(:,:)
     CALL Write_Ncdf_var('UXBAR',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = UYBAR(:,:)
     CALL Write_Ncdf_var('UYBAR',dimnames,TRIM(filout1),tab,'double')
     DEALLOCATE(tab)
     ! tab 3D
     dimnames1(1)='x'
     dimnames1(2)='y'
     dimnames1(3)='z1'
     ALLOCATE(tab1(nx,ny,nz+nzm))
     tab1(:,:,:) = T(:,:,:)

     CALL Write_Ncdf_var('T',dimnames1,TRIM(filout1),tab1,'double')
     DEALLOCATE(tab1)
     dimnames1(1)='x'
     dimnames1(2)='y'
     dimnames1(3)='z'

     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = UX(:,:,:)

     CALL Write_Ncdf_var('UX',dimnames1,TRIM(filout1),tab1,'double')
     DEALLOCATE(tab1)

     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = UY(:,:,:)

     CALL Write_Ncdf_var('UY',dimnames1,TRIM(filout1),tab1,'double')
     DEALLOCATE(tab1)


     return
   end subroutine write_cptrtemp


   subroutine write_sortie(indic)

     USE netcdf
     USE io_netcdf
     use netcdf_mod

     USE module3D_phy
     USE module_choix

     implicit none

     integer :: indic

     INTEGER :: status
     INTEGER :: ncid
     REAL*8, DIMENSION(:,:), POINTER :: tab
     REAL*8, DIMENSION(:,:,:), POINTER :: tab1
     CHARACTER(len=10) :: namevar
     CHARACTER(len=20),DIMENSION(2) :: dimnames
     CHARACTER(len=20),DIMENSION(3) :: dimnames1
     CHARACTER(5) charint
     real,dimension(NX,NY) :: iFLOT
     real,dimension(NX,NY) :: iFLOTMX
     real,dimension(NX,NY) :: iFLOTMY

     if ( .NOT. Agrif_Root() .AND.                     &
          Agrif_Nbstepint() .NE. (Agrif_Rhot()-1) ) return

     if (indic .LE. 9) then
        write(charint,'(i1)')indic
     elseif ( indic .LE. 99 ) then
        write(charint,'(i2)')indic
     elseif ( indic .LE. 999 ) then
        write(charint,'(i3)')indic
     elseif ( indic .LE. 9999 ) then
        write(charint,'(i4)')indic
     else
        write(charint,'(i5)')indic
     endif
     !
     if ( indic .EQ. 0 ) then

        fil_sortie_S      ='Netcdf/sortie_S.nc'
        fil_sortie_H      ='Netcdf/sortie_H.nc'
        fil_sortie_B      ='Netcdf/sortie_B.nc'
        fil_sortie_Bsoc   ='Netcdf/sortie_Bsoc.nc'
        fil_sortie_Ibase  ='Netcdf/sortie_Ibase.nc'
        fil_sortie_Bmelt  ='Netcdf/sortie_Bmelt.nc'
        fil_sortie_Hwater ='Netcdf/sortie_Hwater.nc'
        fil_sortie_UXbar  ='Netcdf/sortie_UXbar.nc'
        fil_sortie_UYbar  ='Netcdf/sortie_UYbar.nc'
        fil_sortie_T      ='Netcdf/sortie_T.nc'
        fil_sortie_ux     ='Netcdf/sortie_ux.nc'
        fil_sortie_uy     ='Netcdf/sortie_uy.nc'
        fil_sortie_flot   ='Netcdf/sortie_flot.nc'
        fil_sortie_sa     ='Netcdf/sortie_sa.nc'
        fil_sortie_s2a    ='Netcdf/sortie_s2a.nc'
        fil_sortie_btt    ='Netcdf/sortie_btt.nc'

        !         print*,TRIM(fil_sortie_S)
        status  = nf90_create(TRIM(fil_sortie_S),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_H)
        status  = nf90_create(TRIM(fil_sortie_H),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_B)
        status  = nf90_create(TRIM(fil_sortie_B),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_Bsoc)
        status  = nf90_create(TRIM(fil_sortie_Bsoc),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_Ibase)
        status  = nf90_create(TRIM(fil_sortie_Ibase),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_Bmelt)
        status  = nf90_create(TRIM(fil_sortie_Bmelt),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_Hwater)
        status  = nf90_create(TRIM(fil_sortie_Hwater),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_UXbar)
        status  = nf90_create(TRIM(fil_sortie_UXbar),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_UYbar)
        status  = nf90_create(TRIM(fil_sortie_UYbar),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_T)
        status  = nf90_create(TRIM(fil_sortie_T),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_ux)
        status  = nf90_create(TRIM(fil_sortie_ux),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_uy)
        status  = nf90_create(TRIM(fil_sortie_uy),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_flot)
        status  = nf90_create(TRIM(fil_sortie_flot),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_S2A)
        status  = nf90_create(TRIM(fil_sortie_S2A),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_SA)
        status  = nf90_create(TRIM(fil_sortie_SA),NF90_WRITE,ncid)
        status  = nf90_close(ncid)
        !         print*,TRIM(fil_sortie_BTT)
        status  = nf90_create(TRIM(fil_sortie_BTT),NF90_WRITE,ncid)
        status  = nf90_close(ncid)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_S),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_S),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_S),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_S),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_H),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_H),ny)
        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_H),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_H),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_B),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_B),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_B),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_B),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_Bsoc),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_Bsoc),ny)
        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_Bsoc),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_Bsoc),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_Ibase),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_Ibase),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_Ibase),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_Ibase),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_Bmelt),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_Bmelt),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_Bmelt),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_Bmelt),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_Hwater),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_Hwater),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_Hwater),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_Hwater),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_UXbar),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_UXbar),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_UXbar),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_UXbar),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_UYbar),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_UYbar),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_UYbar),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_UYbar),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_T),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_T),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_T),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_T),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_S2A),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_S2A),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_S2A),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_S2A),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_SA),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_SA),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_SA),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_SA),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_BTT),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_BTT),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_BTT),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_BTT),nz+nzm)


        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_ux),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_ux),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_ux),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_ux),nz+nzm)

        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_uy),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_uy),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_uy),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_uy),nz+nzm)


        CALL Write_Ncdf_dim('x',TRIM(fil_sortie_flot),nx)
        CALL Write_Ncdf_dim('y',TRIM(fil_sortie_flot),ny)

        CALL Write_Ncdf_dim('z',TRIM(fil_sortie_flot),nz)
        CALL Write_Ncdf_dim('z1',TRIM(fil_sortie_flot),nz+nzm)

     endif
     !-------------------------------------------------------------------------------
     ! tab 2D
     dimnames(1)='x'
     dimnames(2)='y'

     ALLOCATE(tab(nx,ny))
     tab(:,:) = S(:,:)
     namevar ='S_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_S),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab(nx,ny))
     tab(:,:) = H(:,:)

     namevar ='H_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_H),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab(nx,ny))
     tab(:,:) = B(:,:)
     namevar ='B_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_B),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab(nx,ny))
     tab(:,:) = Bsoc(:,:)

     namevar ='Bsoc_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_Bsoc),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab(nx,ny))
     tab(:,:) = IBase(:,:)

     namevar ='IBase_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_Ibase),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab(nx,ny))
     tab(:,:) = Bmelt(:,:)
     namevar ='Bmelt_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_Bmelt),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab(nx,ny))
     tab(:,:) = Hwater(:,:)

     namevar ='Hwater_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_Hwater),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab(nx,ny))
     tab(:,:) = UXBAR(:,:)

     namevar ='Uxbar_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_UXbar),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab(nx,ny))
     tab(:,:) = UYBAR(:,:)
     namevar ='Uybar_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames,TRIM(fil_sortie_UYbar),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------
     ! tab 3D
     dimnames1(1)='x'
     dimnames1(2)='y'
     dimnames1(3)='z1'
     ALLOCATE(tab1(nx,ny,nz+nzm))
     tab1(:,:,:) = T(:,:,:)

     namevar ='T_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames1,TRIM(fil_sortie_T),tab1,'double')
     DEALLOCATE(tab1)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = SA(:,:,:,n1poly)

     namevar ='SA_'//trim(charint)
     CALL Write_Ncdf_var(namevar,dimnames1,TRIM(fil_sortie_SA),tab1,'double')
     DEALLOCATE(tab1)
     !-------------------------------------------------------------------------------
     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = S2A(:,:,:,n1poly)

     namevar ='S2A_'//trim(charint)
     CALL Write_Ncdf_var(namevar,dimnames1,TRIM(fil_sortie_S2A),tab1,'double')
     DEALLOCATE(tab1)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = BTT(:,:,:,n1poly)
     namevar ='BTT_'//trim(charint)
     CALL Write_Ncdf_var(namevar,dimnames1,TRIM(fil_sortie_BTT),tab1,'double')
     DEALLOCATE(tab1)
     !-------------------------------------------------------------------------------
     dimnames1(1)='x'
     dimnames1(2)='y'
     dimnames1(3)='z'
     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = UX(:,:,:)
     namevar ='Ux_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames1,TRIM(fil_sortie_ux),tab1,'double')
     DEALLOCATE(tab1)
     !-------------------------------------------------------------------------------

     ALLOCATE(tab1(nx,ny,nz))
     tab1(:,:,:) = UY(:,:,:)

     namevar ='Uy_'//trim(charint)
     CALL Write_Ncdf_var(TRIM(namevar),dimnames1,TRIM(fil_sortie_uy),tab1,'double')
     DEALLOCATE(tab1)

     iflot   = 0.
     iflotmx = 0.
     iflotmy = 0.
     do j = 1 , ny
        do i = 1 , nx
           if ( flot(i,j) == .TRUE. ) then
              iflot(i,j) = 1.
           endif
           if ( flotmx(i,j) == .TRUE. ) then
              iflotmx(i,j) = 1.
           endif
           if ( flotmy(i,j) == .TRUE. ) then
              iflotmy(i,j) = 1.
           endif
        enddo
     enddo
     !-------------------------------------------------------------------------------
     dimnames(1)='x'
     dimnames(2)='y'
     ALLOCATE(tab(nx,ny))
     tab(:,:) = iflot(:,:)

     namevar ='Flot_'//trim(charint)
     CALL Write_Ncdf_var(namevar,dimnames,TRIM(fil_sortie_flot),tab,'double')
     DEALLOCATE(tab)
     ALLOCATE(tab(nx,ny))
     tab(:,:) = iflotmx(:,:)
     namevar ='Flotmx_'//trim(charint)
     CALL Write_Ncdf_var(namevar,dimnames,TRIM(fil_sortie_flot),tab,'double')
     DEALLOCATE(tab)

     ALLOCATE(tab(nx,ny))
     tab(:,:) = iflotmy(:,:)
     namevar ='Flotmy_'//trim(charint)
     CALL Write_Ncdf_var(namevar,dimnames,TRIM(fil_sortie_flot),tab,'double')
     DEALLOCATE(tab)
     !-------------------------------------------------------------------------------

     return
   end subroutine write_sortie


      subroutine write_test()

      USE netcdf
      USE io_netcdf
      use netcdf_mod

      USE module3D_phy

      implicit none

      INTEGER :: status
      INTEGER :: ncid
      character(len=1000) :: filout1
      REAL*8, DIMENSION(:,:), POINTER :: tab
      REAL*8, DIMENSION(:,:,:), POINTER :: tab1
      CHARACTER(len=10) :: namevar
      CHARACTER(len=20),DIMENSION(2) :: dimnames
      CHARACTER(len=20),DIMENSION(3) :: dimnames1

      return
      end subroutine write_test
#endif

      subroutine agrif_base_interp2(tabin ,nxc ,nyc ,   &
                                    tabout,nxf,nyf,   &
                                    i_min,j_min,      &
                                    typevar,constant, &
                                    rhox,rhoy)
!
      implicit none
      integer :: nxc , nyc
      real,dimension(nxc,nyc) :: tabin
      integer :: nxf , nyf
      real,dimension(nxf,nyf) :: tabout
      Integer :: i_min,j_min
      character(*) :: typevar
      logical :: constant
      integer :: rhox,rhoy

      real :: cff1
      integer :: ii,jj,i,j
      real,dimension(:),allocatable :: xc1,yc1,xf,yf
       real,dimension(:,:),allocatable :: tabtemp
       real :: dxc,dyc,dxf,dyf
       real :: decalxc,decalyc,decalxf,decalyf

      integer :: ptx,pty
      real :: val(4),xval(4)

      real :: xmin,ymin,x,y
      integer :: ic,jc,itemp,jtemp
      integer :: ival
!
      allocate(xc1(nxc))
      allocate(yc1(nyc))
      allocate(xf(nxf))
      allocate(yf(nyf))
      
      allocate(tabtemp(nxc,nyf))
      
      dxc = 1.
      dyc = 1.
      dxf = 1./rhox
      dyf = 1./rhoy
      
      if (typevar .EQ. 'F') THEN
        ptx = 2
        pty = 2
        decalxc = 0.
        decalyc = 0.
        decalxf = 0.
        decalyf = 0.
      elseif (typevar .EQ. 'T') THEN
        ptx = 3
        pty = 3
        decalxc = dxc/2.
        decalyc = dyc/2.
        decalxf = dxf/2.
        decalyf = dyf/2.
      elseif (typevar .EQ. 'U') THEN
        ptx = 2
        pty = 3
        decalxc = 0.
        decalyc = dyc/2.
        decalxf = 0.
        decalyf = dyf/2.
      elseif (typevar .EQ. 'V') THEN
        ptx = 3
        pty = 2
        decalxc = dxc/2.
        decalyc = 0.
        decalxf = dxf/2.
        decalyf = 0.
      ENDIF
      
      do i=1,nxc
        xc1(i) = 0. + (i-ptx) * dxc + decalxc
      enddo
      
      do j=1,nyc
        yc1(j) = 0. + (j-pty) * dyc + decalyc
      enddo   
      
      
      xmin = (i_min - 1) * dxc
      ymin = (j_min - 1) * dyc
      
      do i=1,nxf
        xf(i) = xmin + (i-ptx) * dxf + decalxf
      enddo
      
      do j=1,nyf
        yf(j) = ymin + (j-pty) * dyf + decalyf
      enddo
            
      if ( constant ) then
         do j = 1,nyf
           do i = 1,nxf
              x = xf(i)
              y = yf(j)
          
              ic = ptx + nint((x-0.-decalxc)/dxc)
              jc = pty + nint((y-0.-decalyc)/dyc)

              tabout(i,j) = tabin(ic,jc)
           enddo
         enddo
      else
!      
      do j = 1,nyf
        do i = 1,nxc
          x = xc1(i)
          y = yf(j)
          
          ic = ptx + nint((x-0.-decalxc)/dxc)
!!!          
          ival = floor((y-0.-decalyc)/dyc) + 1
!      
          if( abs((y-0.-decalyc)/dyc - ival).le.0.0001 )then
             ival = ival
          else
             ival = ival-1
          endif
!!!
          jc = pty + ival
          
          jc = jc - 1
          
          call polint(yc1(jc:jc+3),tabin(ic,jc:jc+3),4,y,tabtemp(i,j))
        enddo
      enddo
      
      do j = 1,nyf
        do i = 1,nxf
          x = xf(i)
          y = yf(j)
!!!          
          ival = floor((x-0.-decalxc)/dxc) + 1
!      
          if( abs((x-0.-decalxc)/dxc - ival).le.0.0001 )then
             ival = ival
          else
             ival = ival-1
          endif
!!!
          itemp = ptx + ival
          jtemp = pty + nint((y-ymin-decalyf)/dyf)
          
          itemp = itemp - 1
          
          val = tabtemp(itemp:itemp+3,jtemp)
          xval = xc1(itemp:itemp+3)
          call polint(xval,val,4,x,tabout(i,j))
          
        enddo
      enddo
      endif
      
      deallocate(xc1,yc1,xf,yf,tabtemp)


      end subroutine agrif_base_interp2   
!
!********************************************************
!   subroutine polint                                        *
!********************************************************
!       
      subroutine polint(xin,valin,n,x,val)
      implicit none
      integer n
      real xin(n), valin(n)
      real x,val
      
      integer ns,i,m
      real  dif,dift
      real c(n),d(n),ho,hp,w,den,dy
      
      ns = 1
      dif = abs(x-xin(1))
      
      do i=1,n
        dift = abs(x-xin(i))
        if (dift < dif) then
          ns = i
          dif = dift
        endif
        c(i) = valin(i)
        d(i) = valin(i)
      enddo
      
      val = valin(ns)
      ns = ns - 1
      
      do m=1,n-1
        do i=1,n-m
          ho = xin(i)-x
          hp = xin(i+m)-x
          w=c(i+1)-d(i)
          den = w/(ho-hp)
          d(i) = hp * den
          c(i) = ho * den
         enddo
         if (2*ns < (n-m)) then
           dy = c(ns+1)
         else
           dy = d(ns)
           ns = ns - 1
         endif
         val = val + dy
        enddo
      end subroutine polint   
!
