!> \file eq_elliptique_mod-0.4.f90
!!Module pour la resolution des equations elliptiques de l'ice shelf
!<

!> \namespace eq_elliptique_mod
!! Module pour la resolution des equations elliptiques de l'ice shelf
!! \author ...
!! \date ...
!! @note Used module
!! @note   - use module3D_phy
!<
module eq_elliptique_mod


! declaration des variables

implicit none

integer :: kl,ku,kdc2    
integer :: ldbmax   ! largeur de bande maximum (taille de la matrice) >= 2kl+ku+1
integer :: nptmax   ! nombre maximum de points dans le domaine


!      parameter (NPTMAX=15000) ! en 40km
!      parameter (NPTMAX=60000) ! en 20km

real,dimension(:,:), allocatable::BDR
real,dimension(:,:), allocatable::MMAT
real,dimension(:),   allocatable::BDRO

!      real BDR(NPTMAX,1),MMAT(LDBMAX,NPTMAX),BDRO(NPTMAX)


CONTAINS

!> SUBROUTINE: initial_matrice
!!@note Routine pour l'initialisation des matrices
!<
subroutine initial_matrice

  USE module3D_phy

! we initialize NPTMAX and LDBMAX
  if (geoplace.eq.'anteis1') then
                                ! for the 40km grid:
     kl=284 ! 2*141
     ku=284
     NPTMAX=39762 ! 40km grid : 2*nx*ny=2*141**2
!     parameter (NPTMAX=39762) ! 40km grid : 2*nx*ny=2*141**2
! On initialise la matrice maximale car remplimat_rescue calcul
! sur tout le domaine
!    kl=171
!    ku=171
!    NPTMAX=2000!10000
  elseif (geoplace.eq.'ant20km') then
                                ! for the 20km grid:
     kl=341
     ku=341
     NPTMAX=60000
  elseif (geoplace(1:5).eq.'hemin') then
     kl=400
     ku=400
     NPTMAX=90000
  else
     kl=nx+ny
     ku=nx+ny
     NPTMAX=2*nx*ny     
  endif
     kdc2=KL+KU+1
     LDBMAX=2*KL+KU+1
     
  if (.not.allocated(BDR)) THEN
     allocate(BDR(NPTMAX,1),stat=err)
     if (err/=0) then
        print *,"Erreur  l'allocation du tableau BDR",err
        stop 4
     end if
  end if
  if (.not.allocated(MMAT)) THEN
     allocate(MMAT(LDBMAX,NPTMAX),stat=err)
     if (err/=0) then
        print *,"Erreur  l'allocation du tableau MMAT",err
        stop 4
     end if
  end if
  if (.not.allocated(BDRO)) THEN
     allocate(BDRO(NPTMAX),stat=err)
     if (err/=0) then
        print *,"Erreur  l'allocation du tableau BDRO",err
        stop 4
     end if
  end if
     
    print*,'LDBMAT',LDBMAX
 print*,'NPTMAT',NPTMAX

 END subroutine INITIAL_MATRICE 

!> SUBROUTINE: definition_matrice
!! Routine pour les definition des matrices
!! \param initi 
!! \param nptmat
!! \param ldbmat
!>
subroutine definition_matrice(initi,nptmat,ldbmat)

USE module3D_phy
    
      logical INITI
      INTEGER NPTMAT
      INTEGER LDBMAT
      
initiation : IF (INITI) THEN
     
        print *,"Erreur mauvais appel a DEFINITION_MATRICE"
        ELSE !initia

!here we recall DEFINITION_MATRICE cause the former matrices
!have become too shallow
!neow allocation with new NPTMAT,LDBMAT sent by remplimat   
   print *,"realocation" 
   print*,'NPTMAT',NPTMAT
   print*,'LDBMAT',LDBMAT
   
   deallocate(BDR,MMAT,BDRO,stat=err)
     if (err/=0) then
        print *,"Erreur  la desallocation des tableau BDR,MMAT,BDRO",err
        stop 4
     end if
  
   if (.not.allocated(BDR)) THEN
     allocate(BDR(NPTMAT,1),stat=err)
     if (err/=0) then
        print *,"Erreur  la re-allocation du tableau BDR",err
        stop 4
     end if
  end if
  if (.not.allocated(MMAT)) THEN
     allocate(MMAT(LDBMAT,NPTMAT),stat=err)
     if (err/=0) then
        print *,"Erreur  la re-allocation du tableau MMAT",err
        stop 4
     end if
  end if
  if (.not.allocated(BDRO)) THEN
     allocate(BDRO(NPTMAT),stat=err)
     if (err/=0) then
        print *,"Erreur  la re-allocation du tableau BDRO",err
        stop 4
     end if
  end if
        ENDIF initiation

END subroutine DEFINITION_MATRICE

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!



END MODULE EQ_ELLIPTIQUE_MOD
