!> \file lect-Ant_CISM_gen_dat.f90
!!Module pour la lecture de la topographie
!<

!> \namespace lect_topo_ant_CISM_gen
!! Module pour la lecture de la topographie
!! \author ...
!! \date ...
!! @note Used module
!! @note   - use module3D_phy
!! @note   - use param_phy_mod
!<

module lect_topo_ant_gen

  use module3D_phy
  use param_phy_mod
  use interface_input
  use io_netcdf_grisli

  character(len=100) :: topo_surf      !< surface file name
  character(len=100) :: correc_surf    !< ice-real correction file name
  character(len=100) :: topo_thick     !< thickness file name
  character(len=100) :: topo_bed       !< bedrock file name
  character(len=100) :: mask_grounded  !< mask file name
  character(len=100) :: longitude      !< longitude file name
  character(len=100) :: latitude       !< latitude file name
  character(len=100) :: heatflux       !< geothermal heat flux
  logical            :: corr_firn      !< eventuellement correction firn (deja fait dans Le Brocq)

!  for a start with a different topography than the reference one (without temperature inside)
  integer            :: istart = 0     !< to read another topography than the present one > 0, (2 when  icptr=1)
  character(len=100) :: surf_start     !< surface file name
  character(len=100) :: thick_start    !< thickness file name
  character(len=100) :: bed_start      !< bed file name
  character(len=100) :: mask_start     !< masque file name

  real,dimension(nx,ny) :: work_tab 
  real*8,dimension(:),pointer        :: Tab1D           ! pointeur vers les tableaux des coordonnees

  character(len=100) :: file_ncdf      !< fichier netcdf issue des fichiers .dat

contains

  subroutine input_topo

    namelist/topo_ant_gen/topo_surf,correc_surf,topo_thick,topo_bed,mask_grounded,longitude,latitude,heatflux
    namelist/topo_ant_startingpoint/surf_start,thick_start,bed_start,mask_start,sealevel

    if (itracebug.eq.1)  call tracebug(' Entree dans routine input_topo')

428 format(A)

    rewind(num_param)                     ! pour revenir au debut du fichier param_list.dat
    read(num_param,topo_ant_gen)

    write(num_rep_42,428)'!___________________________________________________________' 
    write(num_rep_42,428)'!  module  lect_topo_ant_gen                                '
    write(num_rep_42,topo_ant_gen)
    write(num_rep_42,428)'!___________________________________________________________' 


    if ((trim(correc_surf).eq.'no').or.(trim(correc_surf).eq.'NO')) then 
       corr_firn=.false.
    else
       corr_firn=.true.
    end if

    topo_surf     = trim(dirnameinp)//trim(topo_surf)
    correc_surf   = trim(dirnameinp)//trim(correc_surf)
    topo_thick    = trim(dirnameinp)//trim(topo_thick)
    topo_bed      = trim(dirnameinp)//trim(topo_bed)
    mask_grounded = trim(dirnameinp)//trim(mask_grounded)
    longitude     = trim(dirnameinp)//trim(longitude)
    latitude      = trim(dirnameinp)//trim(latitude)
    heatflux      = trim(dirnameinp)//trim(heatflux)


    ! Transformation des fichiers .dat en .nc

    file_ncdf= trim(dirnameinp)//trim(runname)//'.nc'

    ! lecture adaptee aux fichiers ZBL.dat

    call lect_input(1,'Bsoc',1,Bsoc,topo_bed,file_ncdf)
    !call lect_datfile(nx,ny,Bsoc,1,topo_bed)               ! bedrock
    Bsoc0(:,:) = Bsoc(:,:)

    call lect_input(1,'S',1,S,topo_surf,file_ncdf)
    !call lect_datfile(nx,ny,S,1,topo_surf)                 ! surface
    
    call lect_input(1,'H',1,H,topo_thick,file_ncdf)
    !call lect_datfile(nx,ny,H,1,topo_thick)                ! thickness

    if (itracebug.eq.1)  call tracebug(' avant corr firn')

    if (corr_firn) then                                    ! density correction
       !call lect_datfile(nx,ny,work_tab,1,correc_surf)     ! for CISM files but not 
       ! Cat treatment of Le Brocq
       ! already done
       
       call lect_input(1,'work_tab1',1,work_tab,correc_surf,file_ncdf)

       where (H(:,:).gt.0.)
          work_tab(:,:)=min(H(:,:),work_tab(:,:))   
          S(:,:) = S(:,:) - work_tab(:,:)                  ! corrected surface
          H(:,:) = H(:,:) - work_tab(:,:)                  ! corrected thickness
       end where
    end if
    if (itracebug.eq.1)  call tracebug(' apres corr firn')
    H(:,:)=max(0.,H(:,:))
    H0(:,:)= H(:,:)
    S0(:,:)= S(:,:)


    mk0(:,:)=0.
    mk_init(:,:)=0.

    call lect_input(1,'work_tab2',2,work_tab,mask_grounded,file_ncdf)
    !call lect_datfile(nx,ny,work_tab,1,mask_grounded)      ! masque

    ! definition du Mk_init
    !__________________________________________
    !  outcrops         -> -4
    !  ocean libre      -> -3
    !  iles a pb        -> -2
    !  pose avec glace  ->  0
    !  ice streams      ->  1 d'apres Le Brocq
    !  ice rise         ->  2 pas attribue ici 
    !  ice shelves      ->  3

    mk_init(:,:) = work_tab(:,:)

    if (itracebug.eq.1)  call tracebug(' avant worktab')


    ! to remove nasty islands transform them in ocean
!!$    where (mk_init(:,:).eq.-2)    ! remove the test on H .and.(H(:,:).gt.1.))
!!$       H(:,:) =1     
!!$       S(:,:)= coef_Sflot * H(:,:)
!!$       Bsoc(:,:)=min(Bsoc(:,:),-100.)
!!$       B(:,:)= coef_Bflot * H(:,:)
!!$    end where


! test consistency of input data files (flotation, thickness ...)

! Le masque (moyenne ou sampling)  n'est pas forcement exact 
! le calcul de flot est plus fiable et est base sur l'epaisseur

    do j=1,ny
       do i=1,nx
          if ((Bsoc(i,j)+H(i,j)*ro/row -sealevel).lt.0.) then   ! flottaison
             flot(i,j)=.true.

             if (mk_init(i,j).eq.-2) then    ! ile a probleme : on fait une montagne de 100 m
                Bsoc(i,j) = max(Bsoc(i,j),sealevel + 100.)
                S(i,j)    = max(S(i,j),Bsoc(i,j))
                H(i,j)    = S(i,j) - Bsoc(i,j)
                B(i,j)    = Bsoc(i,j)
                ! dans le dragging, drag_centre = beta_limgz

             else

                if (H(i,j).gt.0.) then                ! ice shelf
                   mk_init(i,j) = 3       
                   S(i,j)       = coef_Sflot * H(i,j) + sealevel
                   B(i,j)       = S(i,j) - H(i,j)

                else if (H(i,j).le.0.) then                   
                   if (Bsoc(i,j).lt.sealevel) then
                      mk_init(i,j) = -3                ! ocean libre
                      H(i,j)       =  0.
                      S(i,j)       =  0.
                      B(i,j)       =  0.
                   else
                      mk_init(i,j) = -4                ! outcrop
                      H(i,j)       =  0.
                      S(i,j)       = Bsoc(i,j)
                      B(i,j)       = Bsoc(i,j)
                   end if

                end if
             endif

          else                                       ! point pose
             flot(i,j)=.false.
             if ( (mk_init(i,j).eq.-4).or.(mk_init(i,j).eq.1).or.(mk_init(i,j).eq.2) ) then 
                ! outcrop ou ice streams ou ice rise : on ne change pas

             else if (mk_init(i,j).eq.-2) then    ! ile a probleme : on fait une montagne de 100 m
                                                  ! dans le dragging, drag_centre = betalimgz

                Bsoc(i,j) = max(Bsoc(i,j),sealevel + 100.)
                S(i,j)    = min(S(i,j),Bsoc(i,j))
                H(i,j)    = S(i,j) - Bsoc(i,j)
                B(i,j)    = Bsoc(i,j)
             else 
                if (H(i,j).gt.0) then 
                   mk_init(i,j) = 0               ! cas general du point pose
                else
                   mk_init(i,j) = -4              ! pas de glace -> outcrop
                end if
             end if

                                                  ! dans tous les cas  poses           
             S(i,j)    = Bsoc(i,j) + H(i,j)
             B(i,j)    = Bsoc(i,j)

          endif
       enddo
    enddo


! on laisse ce passage mais il ne devrait plus y avoir de probleme.

    do J=1,ny
       do i=1,nx
          if ((abs(mk_init(i,j)).ne.3)) then         ! points poses

             if (abs((S(i,j)-Bsoc(i,j))-H(i,j)).ge.1.e-2) then
                write(6,*) i,j,mk_init(i,j),'S-B =! H',(S(i,j)-Bsoc(i,j))-H(i,j)
             endif

             if (coef_Sflot*Bsoc(i,j)/coef_Bflot.gt.S(i,j)) then
                write(6,*) i,j,mk_init(i,j),'flotte S:',coef_Sflot*Bsoc(i,j)/coef_Bflot, S(i,j)
             end if

          else if ((abs(mk_init(i,j)).eq.3)) then   ! points flottants
             if (abs(S(i,j) - coef_Sflot * H(i,j)).gt.1.) then
                write(6,*)
                write(6,*) i,j,mk_init(i,j),'mauvaise flottaison',S(i,j) - coef_Sflot * H(i,j),'S H S2 B Bsoc B2'
                write(6,'(7(f0.2,2x))') S(i,j), H(i,j), coef_Sflot * H(i,j), B(i,j), Bsoc(i,j),coef_Bflot* H(i,j)
                
             endif

             if (coef_Bflot* H(i,j).lt.Bsoc(i,j)-1.) then
                write(6,*) i,j,mk_init(i,j),'frotte B: ',coef_Bflot* H(i,j), Bsoc(i,j)
             end if


          end if
       end do
    end do
    Bsoc0(:,:) = Bsoc(:,:)
    H0(:,:)= H(:,:)
    S0(:,:)= S(:,:)

    if (itracebug.eq.1)  call tracebug(' apres worktab')

! ecriture des variables apres consistency
!!$do j=1,ny
!!$   do i=1,nx
!!$      write(122,*) S(i,j),H(i,j),Bsoc(i,j),B(i,j),Mk_init(i,j)
!!$   end do
!!$end do



    ! pour l'Antarctique masque mko vrai partout (version 2006)
    MK0(:,:)=1  ! mis apres la lecture cptr


other_start:    if (istart.ge.1) then                 ! restart d'une nouvelle topo

    rewind(num_param)                     ! pour revenir au debut du fichier param_list.dat
    read(num_param,topo_ant_startingpoint)

    write(num_rep_42,428)'!___________________________________________________________' 
    write(num_rep_42,428)'!  module  lect_topo_ant_gen : partir differente topo       '
    write(num_rep_42,topo_ant_gen)
    write(num_rep_42,428)'!___________________________________________________________' 

    surf_start   = trim(dirnameinp)//trim(surf_start)
    thick_start  = trim(dirnameinp)//trim(thick_start)
    bed_start    = trim(dirnameinp)//trim(bed_start)
    mask_start   = trim(dirnameinp)//trim(mask_start)

 endif other_start

    if (istart.eq.1) then     
  
    call lect_input(1,'S',1,S,surf_start,file_ncdf)
    call lect_input(1,'H',1,H,thick_start,file_ncdf)
    call lect_input(1,'Bsoc',1,H,thick_start,file_ncdf)
    call lect_input(1,'work_tab1',1,work_tab,mask_start,file_ncdf)

    mk_init(:,:) = work_tab(:,:)

    where ( mk_init(:,:).eq.1)
       flot(:,:) = .False.
    elsewhere
       flot(:,:) = .True.
    end where

 else if (istart.eq.2) then                     ! juste lecture du masque

   if (itracebug.eq.1)  call tracebug(' dans test istart = 2')

    call lect_input(1,'work_tab1',1,work_tab,mask_start,file_ncdf)

    mk_init(:,:) = work_tab(:,:)
   if (itracebug.eq.1)  call tracebug(' dans test istart = 2')
    where ( mk_init(:,:).eq.1)
       flot(:,:) = .False.
    elsewhere
       flot(:,:) = .True.
    end where


 end if 


    ! le calcul des courbures du socle devrait etre dans le dragging
   if (itracebug.eq.1)  call tracebug(' avant lecture lat long')
    call lect_input(1,'Xlong_1',1,Xlong,longitude,file_ncdf)
    !call lect_datfile(nx,ny,Xlong,1,longitude)      ! lecture des coordonnes geographiques
    
    call lect_input(1,'Ylat_1',1,Ylat,latitude,file_ncdf)
    !call lect_datfile(nx,ny,Ylat,1,latitude)        ! 

   if (itracebug.eq.1)  call tracebug(' apres lecture lat long')

! xmin ymin xmax ymax
!------------------------------------------------
! dans un des fichier grd, lire les xmin et xmax
! Il faudra que ce fichier au moins aie les bonnes valeurs

   write(6,*) 'topo_thick',topo_thick
   call Read_Ncdf_var('x',trim(topo_thick),Tab1D)    ! lit le tableau x
   xmin = minval(Tab1D(:))
   xmax = maxval(Tab1d(:))

   call Read_Ncdf_var('y',trim(topo_thick),Tab1D)    ! lit le tableau y
   ymin = minval(Tab1D(:))
   ymax = maxval(Tab1d(:))

 

!   longitude =trim(dirnameinp)//'long_ZBL_LBq_15km.dat'

!    open(22,file=longitude)                         ! juste pour avoir l'en tete
!    read(22,*) i,i,xmin,xmax,ymin,ymax
    xmin=xmin/1000.
    ymin=ymin/1000.
    xmax=xmax/1000.
    ymax=ymax/1000.
!    close(22)
!   write(6,*) 'bornes domaine a la lecture',xmin,xmax,ymin,ymax


    !     lecture du fichier de reference pour le calcul du niveau des mers : etat actuel
    !     pas de version correcte pour l'instant -> valeurs initiales
    S_sealev(:,:) = S0(:,:)
    H_sealev(:,:) = H0(:,:)
    B_sealev(:,:) = Bsoc(:,:)
    M_sealev(:,:) = Mk0(:,:)

    if (itracebug.eq.1)  call tracebug(' avant lecture ghf')

    call lect_input(1,'ghf',1,ghf,heatflux,file_ncdf)
    !call lect_datfile(nx,ny,ghf,1,heatflux)

    if (itracebug.eq.1)  call tracebug(' apres lecture ghf')

    ! for the purposes of SeaRISE, GTHF should be capped at a value of -0.07 w/m^2,
    ! and the field provided in the netCDF file does not include these updated values

    ghf(:,:)=min(ghf(:,:),70.)           ! pour sea rise

    ! pour passer les flux des W/m2 au J/m2/an      
    ghf(:,:)=-SECYEAR*ghf(:,:)/1000.     ! /1000. parce que les flux sont donnes en mW/m2
    ghf0(:,:)=ghf(:,:)


  end subroutine input_topo


end module lect_topo_ant_gen

